@@ -58,6 +58,7 @@
  */
 
 extern dispatch* global_dispatch;
+static player_equalizer* global_player_equalizer = NULL;
 
 /*
  * player_eq_node
@@ -223,7 +224,7 @@ public:
         std::memcpy(vp[0], viewport, 4 * sizeof(int));
         std::memcpy(vp[1], viewport, 4 * sizeof(int));
         video_output::display_current_frame(0, true, mono_right_instead_of_left,
-                x, y, w, h, vp, my_tex_coords);
+                x, y, w, h, vp, my_tex_coords, 0, 0, parameters::mode_mono_left);
     }
 };
 
@@ -459,9 +460,11 @@ public:
         // Run player steps until we are told to do something
         bool more_steps;
         do {
-            global_dispatch->get_player()->step(&more_steps, &_eq_frame_data.seek_to,
+            global_player_equalizer->step(&more_steps, &_eq_frame_data.seek_to,
                     &_eq_frame_data.prep_frame, &_eq_frame_data.drop_frame, &_eq_frame_data.display_frame);
             dispatch::process_all_events();
+            if (!dispatch::playing())
+                more_steps = false;
         }
         while (more_steps
                 && _eq_frame_data.seek_to == -1
@@ -469,12 +472,12 @@ public:
                 && !_eq_frame_data.drop_frame
                 && !_eq_frame_data.display_frame
                 && !dispatch::pausing());
-        if (!more_steps)
-        {
+        if (!more_steps) {
             this->exit();
+            return 0;
         }
         // Update the video state for all (it might have changed via handleEvent())
-        _eq_frame_data.subtitle = global_dispatch->get_player()->get_subtitle_box();
+        _eq_frame_data.subtitle = global_player_equalizer->get_subtitle_box();
         _eq_frame_data.dispatch_state = global_dispatch->save_state();
         // Find region of canvas to use, depending on the video aspect ratio and zoom level
         float aspect_ratio = dispatch::media_input()->video_frame_template().aspect_ratio;
@@ -565,6 +568,7 @@ public:
                 controller::send_cmd(command::toggle_play);
                 break;
             case 'e':
+            case eq::KC_F7:
                 controller::send_cmd(command::toggle_stereo_mode_swap);
                 break;
             case 'f':
@@ -878,7 +882,7 @@ public:
     const video_frame &get_video_frame()
     {
         if (_is_app_node)
-            return global_dispatch->get_player()->get_video_frame();
+            return global_player_equalizer->get_video_frame();
         else
             return _player.get_video_frame();
     }
@@ -1090,8 +1094,6 @@ public:
  * player_equalizer
  */
 
-static player_equalizer* global_player_equalizer = NULL;
-
 player_equalizer::player_equalizer(int *argc, char *argv[], bool flat_screen) :
     player(), _flat_screen(flat_screen)
 {
@@ -1116,6 +1118,9 @@ player_equalizer::player_equalizer(int *argc, char *argv[], bool flat_screen) :
 
 player_equalizer::~player_equalizer()
 {
+    eq::releaseConfig(_config);
+    eq::exit();
+    exitErrors();
     delete _node_factory;
     global_player_equalizer = NULL;
 }
@@ -1128,20 +1133,36 @@ void player_equalizer::open()
     }
 }
 
-void player_equalizer::close()
+static bool global_quit_request;
+
+class eq_quit_controller : public controller
 {
-    _config->exit();
-    eq::releaseConfig(_config);
-    eq::exit();
-    exitErrors();
-}
+    virtual void receive_notification(const notification& note)
+    {
+        if (note.type == notification::quit)
+            global_quit_request = true;
+    }
+};
 
 void player_equalizer::mainloop()
 {
-    assert(global_player_equalizer);
-    eq_config* config = global_player_equalizer->_config;
-    while (config->isRunning()) {
-        config->startFrame();
-        config->finishFrame();
+    global_quit_request = false;
+    eq_quit_controller qc;
+    for (;;) {
+        if (!global_player_equalizer) {
+            dispatch::step();
+            dispatch::process_all_events();
+            if (global_quit_request)
+                return;
+        }
+        if (global_player_equalizer) {
+            eq_config* config = global_player_equalizer->_config;
+            while (config->isRunning()) {
+                config->startFrame();
+                config->finishFrame();
+            }
+            global_dispatch->stop_eq_player();
+            delete global_player_equalizer;
+        }
     }
 }
