@@ -253,7 +253,7 @@ static void compute_viewport_and_tex_coords(int vp[4], float tc[4][2],
         float src_ar, int w, int h, float dst_w, float dst_h, float dst_ar,
         float crop_ar, float zoom)
 {
-    std::memcpy(tc, full_tex_coords[0], sizeof(tc));
+    std::memcpy(tc, full_tex_coords[0], 4 * 2 * sizeof(float));
     if (crop_ar > 0.0f) {
         if (src_ar >= crop_ar) {
             float cutoff = (1.0f - crop_ar / src_ar) / 2.0f;
@@ -581,7 +581,7 @@ void video_output::init()
 {
     if (!_initialized) {
 #if HAVE_LIBXNVCTRL
-        _nv_sdi_output->init(_params.sdi_output_format());
+        _nv_sdi_output->init(dispatch::parameters().sdi_output_format());
 #endif // HAVE_LIBXNVCTRL
         _full_viewport[0] = -1;
         _full_viewport[1] = -1;
@@ -766,16 +766,18 @@ void video_output::subtitle_deinit(int index)
     _subtitle_updater->reset();
 }
 
-void video_output::color_init(int index, const video_frame &frame)
+void video_output::color_init(int index, const parameters& params, const video_frame &frame)
 {
     xglCheckError(HERE);
     glGenFramebuffersEXT(1, &_color_fbo);
+    std::string quality_str;
     std::string layout_str;
     std::string color_space_str;
     std::string value_range_str;
     std::string storage_str;
     std::string chroma_offset_x_str;
     std::string chroma_offset_y_str;
+    quality_str = str::from(params.quality());
     if (frame.layout == video_frame::bgra32) {
         layout_str = "layout_bgra32";
         color_space_str = "color_space_srgb";
@@ -825,13 +827,17 @@ void video_output::color_init(int index, const video_frame &frame)
             }
         }
     }
-    // XXX: Hack: work around broken SRGB texture implementations
-    if (!srgb8_textures_are_color_renderable() || std::getenv("SRGB_TEXTURES_ARE_BROKEN")) {
-        msg::dbg("Avoiding broken SRGB texture implementation.");
+    if (params.quality() == 0) {
+        storage_str = "storage_srgb";   // SRGB data in GL_RGB8 texture.
+    } else if (storage_str == "storage_srgb"
+            && (!glewIsSupported("GL_EXT_texture_sRGB")
+                || std::getenv("SRGB_TEXTURES_ARE_BROKEN") // XXX: Hack: work around broken SRGB texture implementations
+                || !srgb8_textures_are_color_renderable())) {
         storage_str = "storage_linear_rgb";
     }
 
     std::string color_fs_src(VIDEO_OUTPUT_COLOR_FS_GLSL_STR);
+    str::replace(color_fs_src, "$quality", quality_str);
     str::replace(color_fs_src, "$layout", layout_str);
     str::replace(color_fs_src, "$color_space", color_space_str);
     str::replace(color_fs_src, "$value_range", value_range_str);
@@ -848,10 +854,13 @@ void video_output::color_init(int index, const video_frame &frame)
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
         glTexImage2D(GL_TEXTURE_2D, 0,
-                storage_str == "storage_srgb" ? GL_SRGB8 : GL_RGB16,
+                params.quality() == 0 ? GL_RGB
+                : storage_str == "storage_srgb" ? GL_SRGB8 : GL_RGB16,
                 frame.width, frame.height, 0, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, NULL);
     }
     xglCheckError(HERE);
+    _color_last_params[index] = params;
+    _color_last_frame[index] = frame;
 }
 
 void video_output::color_deinit(int index)
@@ -869,13 +878,15 @@ void video_output::color_deinit(int index)
             _color_tex[index][i] = 0;
         }
     }
+    _color_last_params[index] = parameters();
     _color_last_frame[index] = video_frame();
     xglCheckError(HERE);
 }
 
-bool video_output::color_is_compatible(int index, const video_frame &current_frame)
+bool video_output::color_is_compatible(int index, const parameters& params, const video_frame &current_frame)
 {
-    return (_color_last_frame[index].width == current_frame.width
+    return (_color_last_params[index].quality() == params.quality()
+            && _color_last_frame[index].width == current_frame.width
             && _color_last_frame[index].height == current_frame.height
             && _color_last_frame[index].layout == current_frame.layout
             && _color_last_frame[index].color_space == current_frame.color_space
@@ -887,6 +898,7 @@ bool video_output::color_is_compatible(int index, const video_frame &current_fra
 void video_output::render_init()
 {
     xglCheckError(HERE);
+    std::string quality_str = str::from(_render_params.quality());
     std::string mode_str = (
             _render_params.stereo_mode() == parameters::mode_even_odd_rows ? "mode_even_odd_rows"
             : _render_params.stereo_mode() == parameters::mode_even_odd_columns ? "mode_even_odd_columns"
@@ -910,6 +922,7 @@ void video_output::render_init()
     std::string coloradjust_str = (render_needs_coloradjust(_render_params) ? "coloradjust_enabled" : "coloradjust_disabled");
     std::string ghostbust_str = (render_needs_ghostbust(_render_params) ? "ghostbust_enabled" : "ghostbust_disabled");
     std::string render_fs_src(VIDEO_OUTPUT_RENDER_FS_GLSL_STR);
+    str::replace(render_fs_src, "$quality", quality_str);
     str::replace(render_fs_src, "$mode", mode_str);
     str::replace(render_fs_src, "$subtitle", subtitle_str);
     str::replace(render_fs_src, "$coloradjust", coloradjust_str);
@@ -989,7 +1002,8 @@ bool video_output::render_needs_ghostbust(const parameters& params)
 
 bool video_output::render_is_compatible()
 {
-    return (_render_last_params.stereo_mode() == _render_params.stereo_mode()
+    return (_render_last_params.quality() == _render_params.quality()
+            && _render_last_params.stereo_mode() == _render_params.stereo_mode()
             && render_needs_subtitle(_render_last_params) == render_needs_subtitle(_render_params)
             && render_needs_coloradjust(_render_last_params) == render_needs_coloradjust(_render_params)
             && render_needs_ghostbust(_render_last_params) == render_needs_ghostbust(_render_params));
@@ -1141,6 +1155,10 @@ void video_output::display_current_frame(
     */
 
     _render_params = dispatch::parameters();
+    // The quality parameter must always be consistent with the color conversion step
+    // so that we avoid a mismatch between SRGB/RGB linearization/delinearization.
+    _render_params.set_quality(_color_last_params[_active_index].quality());
+
     _render_params.set_stereo_mode(stereo_mode);
     bool context_needs_stereo = (_render_params.stereo_mode() == parameters::mode_stereo);
     if (context_needs_stereo != context_is_stereo()) {
@@ -1242,13 +1260,17 @@ void video_output::display_current_frame(
     }
     glUniform1i(glGetUniformLocation(_render_prg, "rgb_l"), 0);
     glUniform1i(glGetUniformLocation(_render_prg, "rgb_r"), 1);
-    glUniform1f(glGetUniformLocation(_render_prg, "parallax"), _render_params.parallax() * 0.05f);
+    glUniform1f(glGetUniformLocation(_render_prg, "parallax"),
+            _render_params.parallax() * 0.05f
+            * (_render_params.stereo_mode_swap() ? -1 : +1));
     if (render_needs_subtitle(_render_params)) {
         glActiveTexture(GL_TEXTURE2);
         glBindTexture(GL_TEXTURE_2D, (_subtitle[_active_index].is_valid()
                     ? _subtitle_tex[_active_index] : _render_dummy_tex));
         glUniform1i(glGetUniformLocation(_render_prg, "subtitle"), 2);
-        glUniform1f(glGetUniformLocation(_render_prg, "subtitle_parallax"), _render_params.subtitle_parallax() * 0.05f);
+        glUniform1f(glGetUniformLocation(_render_prg, "subtitle_parallax"),
+                _render_params.subtitle_parallax() * 0.05f
+                * (_render_params.stereo_mode_swap() ? -1 : +1));
     }
     if (render_needs_coloradjust(_render_params)) {
         float color_matrix[16];
@@ -1511,10 +1533,9 @@ void video_output::prepare_next_frame(const video_frame &frame, const subtitle_b
 
     /* Step 2: color-correction */
 
-    if (!_color_prg[index] || !color_is_compatible(index, frame)) {
+    if (!_color_prg[index] || !color_is_compatible(index, dispatch::parameters(), frame)) {
         color_deinit(index);
-        color_init(index, frame);
-        _color_last_frame[index] = frame;
+        color_init(index, dispatch::parameters(), frame);
     }
     int left = 0;
     int right = (frame.stereo_layout == parameters::layout_mono ? 0 : 1);
