@@ -47,6 +47,10 @@
 # include <X11/Xlib.h>
 # include <GL/glxew.h>
 #endif
+#ifdef Q_WS_MAC
+#include <IOKit/pwr_mgt/IOPMLib.h>
+#include <AvailabilityMacros.h>
+#endif
 
 #include "gettext.h"
 #define _(string) gettext(string)
@@ -100,7 +104,8 @@ void gl_thread::activate_next_frame()
     _wait_mutex.lock();
     _action_finished = false;
     _action_activate = true;
-    _wait_cond.wait(_wait_mutex);
+    while(_action_activate)
+        _wait_cond.wait(_wait_mutex);
     _action_finished = true;
     _wait_mutex.unlock();
 }
@@ -114,7 +119,8 @@ void gl_thread::prepare_next_frame(const video_frame &frame, const subtitle_box
     _next_frame = frame;
     _action_finished = false;
     _action_prepare = true;
-    _wait_cond.wait(_wait_mutex);
+    while(_action_prepare)
+        _wait_cond.wait(_wait_mutex);
     _action_finished = true;
     _wait_mutex.unlock();
 }
@@ -145,32 +151,66 @@ void gl_thread::run()
             // can only switch video frames every other output frame.
             if (dispatch::parameters().stereo_mode() != parameters::mode_alternating
                     || _display_frameno % 2 == 0) {
+                _wait_mutex.lock();
                 if (_action_activate) {
-                    _vo_qt->video_output::activate_next_frame();
+                    try {
+                        _vo_qt->video_output::activate_next_frame();
+                    }
+                    catch (std::exception& e) {
+                        _e = e;
+                        _render = false;
+                        _failure = true;
+                    }
                     _action_activate = false;
                     _wait_cond.wake_one();
                     _redisplay = true;
                 }
+                _wait_mutex.unlock();
             }
+            if (_failure)
+                break;
+            _wait_mutex.lock();
             if (_action_prepare) {
-                _vo_qt->video_output::prepare_next_frame(_next_frame, _next_subtitle);
+                try {
+                    _vo_qt->video_output::prepare_next_frame(_next_frame, _next_subtitle);
+                }
+                catch (std::exception& e) {
+                    _e = e;
+                    _render = false;
+                    _failure = true;
+                }
                 _action_prepare = false;
                 _wait_cond.wake_one();
             }
+            _wait_mutex.unlock();
+            if (_failure)
+                break;
             if (_w > 0 && _h > 0
                     && (_vo_qt->full_display_width() != _w
                         || _vo_qt->full_display_height() != _h)) {
                 _vo_qt->reshape(_w, _h);
                 _redisplay = true;
             }
-            if (_redisplay || dispatch::parameters().stereo_mode() == parameters::mode_alternating) {
+            // In alternating mode, we always need to redisplay
+            if (dispatch::parameters().stereo_mode() == parameters::mode_alternating)
+                _redisplay = true;
+            // If DLP 3-D Ready Sync is active, we always need to redisplay
+            if (dispatch::parameters().fullscreen() && dispatch::parameters().fullscreen_3d_ready_sync()
+                    && (dispatch::parameters().stereo_mode() == parameters::mode_left_right
+                        || dispatch::parameters().stereo_mode() == parameters::mode_left_right_half
+                        || dispatch::parameters().stereo_mode() == parameters::mode_top_bottom
+                        || dispatch::parameters().stereo_mode() == parameters::mode_top_bottom_half
+                        || dispatch::parameters().stereo_mode() == parameters::mode_alternating))
+                _redisplay = true;
+            // Redisplay if necessary
+            if (_redisplay) {
                 _redisplay = false;
                 _vo_qt->display_current_frame(_display_frameno);
 #if HAVE_LIBXNVCTRL
                 _vo_qt->sdi_output(_display_frameno);
 #endif // HAVE_LIBXNVCTRL
                 _vo_qt_widget->swapBuffers();
-            } else {
+            } else if (!dispatch::parameters().benchmark()) {
                 // do not busy loop
                 usleep(1000);
             }
@@ -181,10 +221,15 @@ void gl_thread::run()
         _render = false;
         _failure = true;
     }
+    _wait_mutex.lock();
     if (_action_activate || _action_prepare) {
-        while (!_action_finished)
+        while (!_action_finished) {
             _wait_cond.wake_one();
+            _wait_mutex.unlock();
+            _wait_mutex.lock();
+     }
     }
+    _wait_mutex.unlock();
     _vo_qt_widget->doneCurrent();
 }
 
@@ -566,6 +611,7 @@ void video_output_qt::init()
         create_widget();
         _widget->makeCurrent();
         set_opengl_versions();
+        glewExperimental = GL_TRUE;
         GLenum err = glewInit();
 #ifdef Q_WS_X11
         if (err == GLEW_OK)
@@ -578,10 +624,24 @@ void video_output_qt::init()
             throw exc(str::asprintf(_("Cannot initialize GLEW: %s"),
                         reinterpret_cast<const char *>(glewGetErrorString(err))));
         }
-        if (!glewIsSupported("GL_VERSION_2_1 GL_EXT_framebuffer_object"))
+        /* We essentially use OpenGL 2.1 + FBOs. Instead of checking for these two,
+         * we check for GL 1.3 + a larger number of extensions, so that GL implementations
+         * that do not fully support 2.1 still work. */
+        if (!glewIsSupported("GL_VERSION_1_3 " // at least OpenGL 1.3 is reasonable
+                    // GL_VERSION_2_0:
+                    "GL_ARB_shader_objects GL_ARB_fragment_shader "
+                    "GL_ARB_texture_non_power_of_two "
+                    // GL_VERSION_2_1:
+                    "GL_ARB_pixel_buffer_object "
+                    // FBOs. Some drivers (e.g. GeForce 6600 on Mac) only provide the EXT
+                    // version, though an ARB version exists
+                    "GL_EXT_framebuffer_object"
+                    // SRGB extensions.
+                    // GL_EXT_texture_sRGB is optional (see video_output.cpp for the test)
+                    // GL_ARB_framebuffer_sRGB is not needed (see video_output.cpp)
+                    ))
         {
-            throw exc(std::string(_("This OpenGL implementation does not support "
-                            "OpenGL 2.1 and framebuffer objects.")));
+            throw exc(std::string(_("This OpenGL implementation does not support required features.")));
         }
         video_output::init();
         video_output::clear();
@@ -782,7 +842,13 @@ void video_output_qt::suspend_screensaver()
 #elif defined(Q_WS_WIN)
     /* TODO */
 #elif defined(Q_WS_MAC)
-    /* TODO */
+# if defined (MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
+    // API introduced in 10.6
+    IOPMAssertionCreateWithName(kIOPMAssertionTypeNoDisplaySleep, kIOPMAssertionLevelOn, CFSTR ("Bino"), &_disableDisplaySleepAssertion);
+# else
+    // API introduced in OSX 10.5, deprecated in 10.6
+    IOPMAssertionCreate(kIOPMAssertionTypeNoDisplaySleep, kIOPMAssertionLevelOn, &_disableDisplaySleepAssertion);
+# endif
 #endif
 }
 
@@ -797,7 +863,7 @@ void video_output_qt::resume_screensaver()
 #elif defined(Q_WS_WIN)
     /* TODO */
 #elif defined(Q_WS_MAC)
-    /* TODO */
+    IOPMAssertionRelease(_disableDisplaySleepAssertion);
 #endif
 }
 
@@ -865,6 +931,9 @@ void video_output_qt::center()
 void video_output_qt::enter_fullscreen()
 {
     if (!_fullscreen) {
+#ifdef Q_WS_MAC
+        _widget->stop_rendering();
+#endif
         // If the container is a window, we save its geometry here so that
         // we can restore it later.
         if (!_container_is_external)
@@ -928,12 +997,18 @@ void video_output_qt::enter_fullscreen()
             _screensaver_inhibited = true;
         }
         _fullscreen = true;
+#ifdef Q_WS_MAC
+        _widget->start_rendering();
+#endif
     }
 }
 
 void video_output_qt::exit_fullscreen()
 {
     if (_fullscreen) {
+#ifdef Q_WS_MAC
+        _widget->stop_rendering();
+#endif
         // Resume the screensaver before disabling fullscreen, so that our window ID
         // still represents the fullscreen window and was the same when suspending the screensaver.
         if (_screensaver_inhibited) {
@@ -955,6 +1030,9 @@ void video_output_qt::exit_fullscreen()
         _container_widget->raise();
         _container_widget->grab_focus();
         _fullscreen = false;
+#ifdef Q_WS_MAC
+        _widget->start_rendering();
+#endif
     }
 }
 
@@ -994,7 +1072,8 @@ void video_output_qt::receive_notification(const notification& note)
 {
     /* Redisplay if a parameter was changed that affects the video display. */
     if (dispatch::playing()
-            && (note.type == notification::stereo_mode
+            && (note.type == notification::quality
+                || note.type == notification::stereo_mode
                 || note.type == notification::stereo_mode_swap
                 || note.type == notification::crosstalk
                 || note.type == notification::fullscreen_flip_left
